# Generic
DEFAULT_EXPORT_FILENAME = "export"

# TimeSeries
TIME_SERIES_FILENAME = "data"
TIME_SERIES_EXT = "csv"

# Component
COMPONENT_VALUES = "values"
COMPONENT_META_PREFIX = "meta_"
COMPONENT_META_LABEL_PREFIX = "label_"
COMPONENT_META_CI_LOWER = "ci_lower"
COMPONENT_META_CI_UPPER = "ci_upper"

# Metadata
METADATA_FILENAME = "meta"
METADATA_EXT = "json"
METADATA_CLASS_LABEL = "label"

# Model
MODEL_TYPE_UNIVARIATE = "univariate"
MODEL_TYPE_MULTIVARIATE = "multivariate"

# Misc. IO
PICKLE_EXT = "pkl"
