from types import SimpleNamespace

COLORS_DICT = {
    "blue_light": "#CADEF7",
    "blue": "#29335C",
    "blue_dark": "#0A2342",
    "green": "#297373",
    "yellow": "#E8C547",
    "red": "#E4572E",
    "red_dark": "#92140C",
    "grey": "#CDD1C4",
}

colors = SimpleNamespace(**COLORS_DICT)
